package es.rediris.forja.jleon.arrivalert;


import java.util.ArrayList;
import java.util.Date;
import java.util.Calendar;
import java.util.Collections;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;


public class BDEvento extends SQLiteOpenHelper {
	
	private Context ctxt;
		
	public BDEvento(Context context) {	
		super(context, "bd_evento", null, 2);
		this.ctxt = context;
	}
	
	//METODOS BDEvento
	
	@Override
	public void onCreate(SQLiteDatabase db) {
		db.execSQL("CREATE TABLE citas ("+	"_id INTEGER PRIMARY KEY AUTOINCREMENT, "+	
	                   "nombre TEXT, descripcion TEXT, fecha LONG, fechaAlerta LONG, kmsTarde FLOAT, estado INTEGER, id_lugar INTEGER)");
		
		
		db.execSQL("CREATE TABLE citas_acciones ("+	"_id INTEGER PRIMARY KEY AUTOINCREMENT, "+	
                "id_cita INTEGER, id_accion INTEGER, id_tipo_accion INTEGER)");	
	}	
	
	
	@Override    
	public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {		
		//actualizacin+eliminacin de tablas
		if (oldVersion == 1 && newVersion==2) {
			db.execSQL("DROP TABLE citas"); 
			db.execSQL("DROP TABLE citas_acciones");
			onCreate(db);
		}
			
	}	
	
	
	
	
	public Evento insertNewCita() {
		
		 Calendar cal = Calendar.getInstance();
		 cal.set(Calendar.MINUTE, 0);
		 cal.set(Calendar.SECOND, 0);
		 cal.set(Calendar.MILLISECOND, 0);
		 cal.add(Calendar.HOUR, 1);
		 Date dw  = cal.getTime();   // fecha por defecto de la alerta: 1 hora despues
		 cal.add(Calendar.HOUR, 1);
		 Date d = cal.getTime();     // fecha por defecto de evento: 2 horas despues
		
		 //nombre
		 String name = this.ctxt != null ? this.ctxt.getString(R.string.nulo) : "";
		 
		 //radio//TODO
		 float kmsDef = Preferencias.getRadio(this.ctxt);
		 
		 // Crear instancia
		 Evento c = new Evento(-1, name, "", d, dw, kmsDef, Evento.ESTADO_PENDIENTE);
		 
		 // insertar bd
		 insertCita(c);
		 
		 return c;		
	}
	
	
	public void insertCita(Evento c) {	
		if(c==null)
			return;
		executeInsert(c);
	}
	
	
	public void deleteCita(int idCita) {	
		if(idCita < 1)
			return;		
				
		// Borramos las referencias
		String sql = getDeleteSqlStringCitas_Acciones(idCita);
		executeSql(sql);
		
		// Borramos la Evento
		sql = getDeleteSqlString(idCita);
		executeSql(sql);
	}
	
	public void deleteAccionEnCita(int idCita, int idAccion, int tipoAccion) {
		if(idCita > 0 && idAccion > 0) {
			String sql = getDeleteSqlStringCitas_Acciones(idCita, idAccion, tipoAccion);
			executeSql(sql);
		}
	}
	
	public void deleteAccionEnCita(Evento c, Alerta a) {
		if(c != null && a != null) {
			deleteAccionEnCita(c.getId(), a.getId(), a.getAlertarID());
		}
	}
	
	public void addAccionEnCita(int idCita, int idAccion, int tipoAccion) {
		if(idCita > 0 && idAccion > 0) {
			String sql = getInsertSqlStringCitas_Acciones(idCita, idAccion, tipoAccion);
			executeSql(sql);
		}
	}
	
	public void addAccionEnCita(Evento c, Alerta a) {
		if(c != null && a != null) {
			addAccionEnCita(c.getId(), a.getId(), a.getAlertarID());
		}
	}
	
	public void deleteCita(Evento c) {	
		if(c != null)
			deleteCita(c.getId());
	}
	
	public void updateCita(Evento c) {	
		if(c==null)
			return;		
				
		String sql = getUpdateSqlString(c);
		executeSql(sql);
	}
	
	public static void updateCita(Context var0, Evento var1) {
	      if(var1 != null) {
	         BDEvento var2 = new BDEvento(var0);
	         var2.updateCita(var1);
	         var2.close();
	      }

	}
	
	public Evento getCita(int idCita) {
		Evento c = null;		
		SQLiteDatabase db = getReadableDatabase();
		
		String sql = "SELECT _id, nombre, descripcion, fecha, fechaAlerta, kmsTarde, estado, id_lugar FROM citas WHERE _id = " + idCita;		
		Cursor cursor = db.rawQuery(sql, null);
				
		// un evento
		if (cursor.moveToNext())
		{
			// ubicacion del evento
			BDUbicacion db_lugares = new BDUbicacion(this.ctxt);//TODO
			int id_lugar = cursor.getInt(7);
			Ubicacion l = null;
			if(id_lugar > 0)
				l = db_lugares.getUbicacion(id_lugar);//TODO
			db_lugares.close();
			
			// Obtenemos la lista de acciones a ejecutar
			ArrayList<Alerta> acciones = this.getAcciones(idCita);
			
			// Creamos la instancia del objeto
			c = new Evento(cursor.getInt(0), 
						  cursor.getString(1),
						  cursor.getString(2),
						  cursor.getLong(3),						  
						  cursor.getLong(4),
						  cursor.getFloat(5),
						  cursor.getInt(6),
						  l,
						  acciones);
		}
		
		cursor.close();
		db.close();
		
		return c;	
	}
	
	
	public static Evento getCita(Context c, int i) {
		  BDEvento bd = new BDEvento(c);
	      Evento e = bd.getCita(i);
	      bd.close();
	      return e;
	}
	
	public ArrayList<Evento> getAllCitas() {
		ArrayList<Evento> citas = new ArrayList<Evento>();
		Evento c = null;		
		SQLiteDatabase db = getReadableDatabase();
		BDUbicacion db_lugares = new BDUbicacion(this.ctxt);//TODO
		
		String sql = "SELECT _id, nombre, descripcion, fecha, fechaAlerta, kmsTarde, estado, id_lugar FROM citas";		
		Cursor cursor = db.rawQuery(sql, null);
				
		// n eventos
		while (cursor.moveToNext())
		{
			// Obtener ubicacion de evento			
			int id_lugar = cursor.getInt(7);
			Ubicacion l = null;
			if(id_lugar > 0)
				l = db_lugares.getUbicacion(id_lugar);//TODO
			
			
			// Obtenemos la lista de acciones a ejecutar
			ArrayList<Alerta> acciones = this.getAcciones(cursor.getInt(0));
			
			// Creamos la instancia del objeto
			c = new Evento(cursor.getInt(0), 
						  cursor.getString(1),
						  cursor.getString(2),
						  cursor.getLong(3),						  
						  cursor.getLong(4),
						  cursor.getFloat(5),
						  cursor.getInt(6),
						  l,
						  acciones);
			
			// anyadimos la Evento a la lista
			citas.add(c);
		}
		
		
		db_lugares.close();//TODO
		cursor.close();
		db.close();
		
		// Ordenamos las citas por nuestro criterio en compareTo
		Collections.sort(citas);
		return citas;
	}
	
	
	
	// metodos interactuan con alertas
	
	public ArrayList<Alerta> getAcciones(int idCita) {
		ArrayList<Alerta> l = new ArrayList<Alerta>();
		if(idCita < 1)
			return l;		
		
		BDAlerta db_acciones = new BDAlerta(this.ctxt);//TODO
		SQLiteDatabase db = getReadableDatabase();
		Alerta  a;
		
		String sql = "SELECT DISTINCT id_accion, id_tipo_accion FROM citas_acciones WHERE id_cita = " + idCita;		
		Cursor cursor = db.rawQuery(sql, null);
		int id_accion, id_tipo_accion;
				
		// solo una ubicacion
		while (cursor.moveToNext()) {
			id_accion = cursor.getInt(0);
			id_tipo_accion = cursor.getInt(1);
			
			switch(id_tipo_accion) {
				case Alerta.ALERTA_IM: 
					a = db_acciones.getAccionMail(id_accion);//TODO
					break;
					
				default:
					a = null;
			}
			
			if(a != null)
				l.add(a);
		}
		
		// Cerramos cursores y bases de datos
		cursor.close();
		db.close();
		db_acciones.close();//TODO		
		
		// ordenamos la lista por nombre
		Collections.sort(l);		
		return l;
	}
	
	
	
	
	// metodos privados
	
	private void executeSql(String sql) {
		SQLiteDatabase db = getWritableDatabase();
		db.execSQL(sql);
		db.close();
	}
	
	private void executeInsert(Evento c) {
		SQLiteDatabase db = getWritableDatabase();
		
		ContentValues cv = new ContentValues();
		cv.put("nombre", c.getNombre());
		cv.put("descripcion", c.getDescripcion());
		cv.put("fecha", c.getFecha().getTime());
		cv.put("fechaAlerta", c.getFechaAlerta().getTime());
		cv.put("kmsTarde", c.getKmsTarde());
		cv.put("estado", c.getEstado());
		cv.put("id_lugar", c.getUbicacion() != null ? c.getUbicacion().getId() : -1);
		
		long id = db.insert("citas", null, cv);
		
		c.setId((int)id);		
		db.close();
	}
	
	private String getDeleteSqlString(int idCita) {
		return String.format("DELETE FROM citas WHERE _id = %s", idCita);
	}
	
	private String getDeleteSqlStringCitas_Acciones(int idCita) {
		return String.format("DELETE FROM citas_acciones WHERE id_cita = %s", idCita);
	}
	
	private String getDeleteSqlStringCitas_Acciones(int idCita, int id_accion, int id_tipo_accion) {
		return String.format("DELETE FROM citas_acciones WHERE id_cita = %s AND id_accion = %s AND id_tipo_accion = %s",
							 idCita,
							 id_accion,
							 id_tipo_accion);
	}
	
	
	private String getUpdateSqlString(Evento c) {
		return String.format("UPDATE citas SET nombre = '%s', descripcion = '%s', fecha =  %s, fechaAlerta = %s, kmsTarde = %s, estado = %s, id_lugar = %s WHERE _id = %s", 
				c.getNombre(), 
				c.getDescripcion(), 
				c.getFecha().getTime(),
				c.getFechaAlerta().getTime(),
				c.getKmsTarde(),
				c.getEstado(),
				c.getUbicacion() != null ? c.getUbicacion().getId() : -1,
				c.getId());
	}
	

	
	private String getInsertSqlStringCitas_Acciones(int idCita, int id_accion, int id_tipo_accion) {
		return String.format("INSERT INTO citas_acciones VALUES (null, %s, %s, %s)",
							 idCita,
							 id_accion,
							 id_tipo_accion);
	}
	
	

	  
	
}
