package es.rediris.forja.jleon.arrivalert;

import android.app.ListActivity;
import android.content.Intent;
import android.os.Bundle;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.Window;
import android.widget.AdapterView;
import android.widget.ListView;
import android.widget.Toast;
import java.util.ArrayList;


public class ConfiguraAlertaActivity extends ListActivity {
	
	private AdaptarElementos adaptelem;
	private Evento ev; 

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		requestWindowFeature(Window.FEATURE_NO_TITLE);
		this.setContentView(R.layout.asocialertas);
		
		/**
		 * Obtener eventos
		 * */
		
		Bundle extras = this.getIntent().getExtras();
		int id = extras == null ? -1 : extras.getInt(Constantes.CTE_EVENTO_ID);
		
		if(id != 0) {
			// Modo editar un registro que ya existe en la bd
			BDEvento db = new BDEvento(this);
			this.ev = db.getCita(id);
			db.close();
			
			if(this.ev != null) {
				
			}
		}

		if(this.ev == null) {
			
			Toast.makeText(this, "ERROR: No tiene ID vlido", Toast.LENGTH_LONG).show();
		}
		
		
		refreshAdaptador();
		
		
	    ListView list = (ListView)this.findViewById(android.R.id.list);
	    registerForContextMenu(list);
	}
	
	private void refreshAdaptador() {
		
		if(this.ev == null) {
			
			Toast.makeText(this, "ERROR: Evento identificado como nulo", Toast.LENGTH_LONG).show();
			return;
		}
		
		// Base de datos actual
		BDEvento bbdd = new BDEvento(this);
        
        // Adaptador  
        ArrayList<Alerta> acciones = bbdd.getAcciones(this.ev.getId()); // acciones para esta Evento
        ArrayList<IListaElementos> lista = new ArrayList<IListaElementos>(acciones);
        this.adaptelem = new AdaptarElementos(this, lista);
        
        // Set the adapter
        setListAdapter(this.adaptelem);
        
        bbdd.close();
	}
	
	
	public void onClickOnAdd(View v) {
		
		if(v.getId() == R.id.btActionMail) {
			Intent i = new Intent(this, AlertaIMActivity.class);
			i.putExtra(Constantes.CTE_MODO, Constantes.CTE_SELECCION);
			this.startActivityForResult(i, Constantes.CTE_ALERTA);
		}
	}
	
	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		
		if(resultCode == RESULT_OK && (requestCode == Constantes.CTE_REG_ALERTA)) {
			refreshAdaptador(); //Refrescar BD
		}
		
		
		else if(resultCode == RESULT_OK && (requestCode == Constantes.CTE_ALERTA)) {
			
			int id_accion = data.getExtras().getInt(Constantes.CTE_ALERTA_ID);
			int tipo_accion = data.getExtras().getInt(Constantes.CTE_ALERTA_IM);
			int id_cita = this.ev.getId();
			
			// hemos de anyadir dicha relación con nuestra Evento			
			BDEvento db = new BDEvento(this);
			db.addAccionEnCita(id_cita, id_accion, tipo_accion);
			db.close();
			
			// refrescamos desde la base de datos
			refreshAdaptador();
		}
	}
	
	
	@Override 
	protected void onListItemClick(ListView listView, View view, int position, long id) {
		super.onListItemClick(listView, view, position, id);
		
		Object o = getListAdapter().getItem(position); 	// Cogemos el item en la posición indicada
		
		// Devolvemos el id y el nombre del lugar seleccionado		
		Alerta a = (Alerta)o;
		editAccion(a);
	}
	
	
	private void editAccion(Alerta a) {
		if(a.getAlertarID() == Alerta.ALERTA_IM) {
			Intent i = new Intent(this, RegistroAlertaIMActivity.class);
			i.putExtra(Constantes.CTE_ALERTA_ID, a.getId());  // editamos la acción con el id especificado
			this.startActivityForResult(i, Constantes.CTE_REG_ALERTA);
		}
		
	}
	
	private void executeNow(Alerta a) {
		if(Preferencias.getEnable(this))
			  a.operarHabilitado(this);
		  else
			  a.operarDeshabilitado(this);
	}
	
	private void deleteFromCitaRelation(Alerta a) {
		// Borramos la fila de la base de datos
		BDEvento bbdd = new BDEvento(this);
        bbdd.deleteAccionEnCita(this.ev.getId(), a.getId(), a.getAlertarID());	        
        bbdd.close();
        
        // refrescamos nuestra lista
        refreshAdaptador();
	}	
	

	/**
	 * Men contextual para cada alerta 
	 * 
	 * **/
	
	
	@Override
	public void onCreateContextMenu(ContextMenu menu, View v, ContextMenuInfo menuInfo) {
		if (v.getId()==android.R.id.list) {
			AdapterView.AdapterContextMenuInfo info = (AdapterView.AdapterContextMenuInfo)menuInfo;
			Alerta a = (Alerta)this.adaptelem.getItem(info.position);
			menu.setHeaderTitle(a.getNombre());
			
			// Add the options
			menu.add(Menu.NONE, 0, 0, R.string.str_edit);
			menu.add(Menu.NONE, 1, 1, R.string.str_execute_now);
			menu.add(Menu.NONE, 2, 2, R.string.str_delete);
		}
	}
	
	
	@Override
	public boolean onContextItemSelected(MenuItem item) {
	  AdapterView.AdapterContextMenuInfo info = (AdapterView.AdapterContextMenuInfo)item.getMenuInfo();
	  int menuItemId  = item.getItemId();
	  Alerta a = (Alerta)this.adaptelem.getItem(info.position);
	  
	  switch(menuItemId) {
		  case 0: editAccion(a); break; // editar
		  case 1: executeNow(a); break; // ejecutar
		  case 2: deleteFromCitaRelation(a); break; // borrar
	  }
	  
	  return true;
	}
}
